@@ -131,11 +131,11 @@ let windows_sleep seconds =
 let min_reserved_fds = 50
 let min_connections = 50
 
-let _ =
+let () =
   lprintf_nl "Starting MLDonkey %s ... " Autoconf.current_version;
   let ulof_old = Unix2.c_getdtablesize () in
   lprintf_nl "Language %s, locale %s, ulimit for open files %d"
-    Charset.default_language Charset.locstr ulof_old;
+    Charset.Locale.default_language Charset.Locale.locale_string ulof_old;
 
   let nofile = Unix2.ml_getrlimit Unix2.RLIMIT_NOFILE in
     if nofile.Unix2.rlim_max > 0 && nofile.Unix2.rlim_max > nofile.Unix2.rlim_cur then
@@ -208,8 +208,8 @@ let _ =
 
   (* Charset conversion self-test *)
   let filename = "abcdefghijklmnopqrstuvwxyz" in
-  let conv_filename = Charset.to_locale filename in
-  if filename <> conv_filename then Charset.conversion_enabled := false;
+  let conv_filename = Charset.Locale.to_locale filename in
+  if filename <> conv_filename then Charset.Locale.conversion_enabled := false;
 
   (try
      ignore (Sys.getenv "MLDONKEY_TEMP")
@@ -1023,6 +1023,7 @@ let web_infos = define_option current_section ["web_infos"]
   'ocl' for file in the ocl format containing overnet peers, or
   'contact.dat' for an contact.dat file containing overnet peers,
   'nodes.gzip' for a fasttrack nodes.gzip,
+  'hublist' for DirectConnect hubs list,
   and period is the period between updates (in hours),
   a period of zero means the file is only loaded once on startup,
   and url is the url of the file to download.
@@ -1030,7 +1031,7 @@ let web_infos = define_option current_section ["web_infos"]
   EXAMPLE:
     web_infos = [
     (\"server.met\", 0, \"http://www.gruk.org/server.met.gz\");
-    (\"hublist\", 0, \"http://dchublist.com/hublist.config.bz2\");
+    (\"hublist\", 0, \"http://dchublist.com/hublist.xml.bz2\");
     (\"guarding.p2p\", 96, \"http://www.bluetack.co.uk/config/level1.gz\");
     (\"ocl\", 24, \"http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl\");
     (\"contact.dat\", 168, \"http://download.overnet.org/contact.dat\");
@@ -1157,8 +1158,8 @@ let mail = define_option current_section ["mail"]
     string_option ""
 
 let add_mail_brackets = define_option current_section ["add_mail_brackets"]
-  "Does your mail-server need <...> around addresses"
-    bool_option false
+  "Set to false if your mail server cannot handle angle-brackets around addresses (RFC 5321)"
+    bool_option true
 
 let filename_in_subject = define_option current_section ["filename_in_subject"]
   "Send filename in mail subject"
@@ -1335,7 +1336,7 @@ let create_dir_mode = define_option current_section ["create_dir_mode"]
     string_option "755"
 
 let create_file_sparse = define_option current_section ["create_file_sparse"]
-  "Create new files as sparse, only valid on MinGW for files on NTFS drives"
+  "Create new files as sparse (not supported on FAT volumes)"
     bool_option true
 
 let hdd_temp_minfree = define_option current_section ["hdd_temp_minfree"]
