@@ -50,7 +50,7 @@ type 'a shared_impl = {
 and 'a shared_ops = {
     mutable op_shared_info : ('a -> GuiTypes.shared_info);
     mutable op_shared_unshare : ('a -> unit);
-    mutable op_shared_state : (CommonTypes.file -> CommonTypes.ui_conn -> string);
+    mutable op_shared_state : ('a -> CommonTypes.ui_conn -> string);
   }
   
 let as_shared  (shared : 'a shared_impl) =
@@ -179,14 +179,15 @@ let shared_codedname s =
 let shared_unshare s =
   let impl = as_shared_impl s in
   shared_remove impl;
-  try impl.impl_shared_ops.op_shared_unshare impl.impl_shared_val with _ -> ()
+  (try impl.impl_shared_ops.op_shared_unshare impl.impl_shared_val with _ -> ());
+  CommonEvent.add_event (Root_console_message_event (Printf.sprintf "INFO: unshared %s\n" impl.impl_shared_fullname))
 
 let shared_state s o =
   let impl = as_shared_impl s in
   try
     match impl.impl_shared_file with
     | None -> ""
-    | Some f -> impl.impl_shared_ops.op_shared_state f o
+    | Some f -> impl.impl_shared_ops.op_shared_state impl.impl_shared_val o
   with _ -> ""
 
 let shared_dir = function
@@ -194,10 +195,8 @@ let shared_dir = function
   | Some sh	-> 
     try
       let impl = as_shared_impl sh in
-      let codedname = impl.impl_shared_codedname in
-      let idx = String.index codedname '/' in
-      String.sub codedname 0 idx
-    with Not_found -> ""
+      Filename.dirname impl.impl_shared_codedname
+    with _ -> ""
 		    
 let shared_prio impl =
   try
