@@ -112,8 +112,9 @@ let eval auth cmd o =
     else cmd in
   let l = String2.tokens cmd in
   match l with
-    [] -> ()
-  | ["longhelp"] | ["??"] ->
+  | [] -> ()
+  | "longhelp"::subs | "??"::subs ->
+      let filter cmd = List.for_all (String2.contains cmd) subs in
       let module M = CommonMessages in
       if o.conn_output = HTML then begin
           Buffer.add_string buf "\\<div class=\\\"cs\\\"\\>";
@@ -125,8 +126,8 @@ let eval auth cmd o =
             ("", "srh", ""); ];
           Buffer.add_string buf "\\</tr\\>";
           html_mods_cntr_init ();
-          List.iter (fun (cmd, _, _, help) ->
-              let ncmd = ref cmd in
+          let show (cmd, _, _, help) =
+              let ncmd = ref cmd in (* why? *)
               let nhelp = ref help in
               Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
@@ -136,9 +137,10 @@ let eval auth cmd o =
                 ("", "sr", "\\<a href=\\\"http://mldonkey.sourceforge.net/" ^ (String2.upp_initial !ncmd) ^
                   "\\\"\\>wiki\\</a\\>"); ];
               Printf.bprintf buf "\\</tr\\>\n";
-          )
-          (List.sort (fun (c1,_, _,_) (c2,_, _,_) -> compare c1 c2)
-            !CommonNetwork.network_commands);
+          in
+          List.iter show 
+            (List.sort (fun (c1,_, _,_) (c2,_, _,_) -> compare c1 c2)
+              (List.filter (fun (c,_,_,_) -> filter c) !CommonNetwork.network_commands));
           Printf.bprintf buf "\\</table\\>\\</div\\>";
           html_mods_table_header buf "helpTable" "results" [];
           Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
@@ -153,15 +155,15 @@ let eval auth cmd o =
           let list = Hashtbl2.to_list2 commands_by_kind in
           let list = List.sort (fun (s1,_) (s2,_) -> compare s1 s2) list in
           List.iter (fun (s,list) ->
-              Printf.bprintf buf "\n   $b%s$n:\n" s;
-              let list = List.sort (fun (s1,_) (s2,_) -> compare s1 s2) !list in
-              List.iter (fun (cmd, help) ->
-                  Printf.bprintf buf "$r%s$n %s\n" cmd help;
-              ) list
+              match List.sort (fun (s1,_) (s2,_) -> compare s1 s2) (List.filter (fun (s,_) -> filter s) !list) with
+              | [] -> ()
+              | list ->
+                Printf.bprintf buf "\n   $b%s$n:\n" s;
+                List.iter (fun (cmd, help) -> Printf.bprintf buf "$r%s$n %s\n" cmd help) list
           ) list;
         end
 
-    | ["help"] | ["?"] ->
+    | ["help"] | ["?"] | ["man"] ->
           let module M = CommonMessages in
            if o.conn_output = HTML then
              begin
@@ -251,7 +253,7 @@ let eval auth cmd o =
                html_mods_td buf [
                  ("", "sr", "Use '$r\\<a href=\\\"submit?q=longhelp\\\"\\>" ^
                    "longhelp\\</a\\>$n' or '$r\\<a href=\\\"submit?q=longhelp\\\"\\>" ^
-                   "??\\</a\\>$n' for all commands."); ];
+                   "??\\</a\\>$n' for all commands. Specify substring to filter."); ];
                Buffer.add_string buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
                html_mods_td buf [
                  ("", "sr", "Use '$rhelp command$n' or '$r? command$n' for help on a command."); ];
@@ -283,14 +285,14 @@ $bGeneral:$n
           $rq$n : quit this interface
 
 Use '$rlonghelp$n' or '$r??$n' for all commands.
+Use '$rlonghelp str$n' or '$r?? str$n' for all commands that contain specified substring.
 Use '$rhelp command$n' or '$r? command$n' for help on a command.
             ";
     | "?" :: args | "help" :: args | "man" :: args ->
           List.iter (fun arg ->
-              List.iter (fun (cmd, _, _, help) ->
-                  if cmd = arg then
-                    Printf.bprintf  buf "%s %s\n" cmd help)
-              !CommonNetwork.network_commands)
+              match List.filter (fun (cmd, _, _, _) -> cmd = arg) !CommonNetwork.network_commands with
+              | [] -> Printf.bprintf buf "Unknown command : %s\n" arg
+              | l -> List.iter (fun (_,_,_,help) -> Printf.bprintf  buf "%s %s\n" arg help) l)
           args
     | one :: two ->
         let cmd, args =
@@ -767,7 +769,7 @@ let read_theme_page page =
     s)
 
 let http_add_gen_header r =
-  add_reply_header r "Server" "MLdonkey";
+  add_reply_header r "Server" ("MLdonkey/"^Autoconf.current_version);
   add_reply_header r "Connection" "close"
 
 let add_gzip_headers r =
@@ -793,6 +795,7 @@ let http_add_text_header r ext =
 let http_add_bin_info_header r clen =
   add_reply_header r "Accept-Ranges" "bytes";
   add_reply_header r "Content-Length" (Printf.sprintf "%d" clen)
+  (* FIXME Content-Length is duplicated *)
 
 let http_add_bin_header r ext clen =
   http_file_type := ext_to_file_type ext;
@@ -820,6 +823,8 @@ let http_send_bin r buf filename =
   in
   let ext = extension_to_file_ext (Filename2.last_extension2 filename) in
   http_add_bin_header r ext (String.length file_to_send);
+  add_reply_header r "Cache-Control" "no-cache";
+  add_reply_header r "Pragma" "no-cache";
   Buffer.add_string buf file_to_send
 
 let http_send_bin_pictures r buf filename =
