@@ -84,35 +84,51 @@ let current_uploaders = ref ([] : BTTypes.client list)
    Everything else will be ok for a second connection to the tracker.
    Be careful to the spelling of this event
   @param f The function used to parse the result of the connection.
-   The function will get a file as an argument (@see
-   get_sources_from_tracker for an example)
-*)
-let connect_trackers file event f =
+   The function will get a file as an argument (@see talk_to_tracker 
+   for an example)
 
-  let args,must_check_delay, downloaded, left =
+  If we have less than !!ask_tracker_threshold sources
+  and if we respect the file_tracker_interval then
+  we really ask sources to the tracker
+*)
+let connect_trackers file event need_sources f =
+
+  (* reset session statistics when sending 'started' event *)
+  if event = "started" then
+  begin
+    file.file_session_uploaded <- Int64.zero;
+    file.file_session_downloaded <- Int64.zero;
+  end;
+
+  let args,must_check_delay, left =
 
     match file.file_swarmer with
       None ->
         begin
           match event with
-          | "started" -> [("event", "started")],true,zero,zero
-          | "stopped" -> [("event", "stopped")],false,zero,zero
-          | _ -> [],true, zero, zero
+          | "started" -> [("event", "started")],true,zero
+          | "stopped" -> [("event", "stopped")],false,zero
+          | _ -> [],true,zero
         end
 
     | Some swarmer ->
         let local_downloaded = CommonSwarming.downloaded swarmer in
         let left = file_size file -- local_downloaded in
         match event with
-        | "completed" -> [("event", "completed")],false,local_downloaded,zero
-        | "started" -> [("event", "started")],true,zero, left
-        | "stopped" -> [("event", "stopped")],false,zero, left
-        | _ -> [],true,local_downloaded, left
+        | "completed" -> [("event", "completed")],false,zero
+        | "started" -> [("event", "started")],true,left
+        | "stopped" -> [("event", "stopped")],false,left
+        | _ -> [],true,left
   in
 
   let args = ("no_peer_id", "1") :: ("compact", "1") :: args in
-  let args = if !!numwant > -1 then
-      ("numwant", string_of_int !!numwant) :: args else args
+  let args = 
+    if not need_sources then
+      ("numwant", "0") :: args
+    else if !!numwant > -1 then
+      ("numwant", string_of_int !!numwant) :: args 
+    else 
+      args
   in
   let args = if !!send_key then
       ("key", Sha1.to_hexa !!client_uid) :: args else args
@@ -124,12 +140,11 @@ let connect_trackers file event f =
     ("info_hash", Sha1.direct_to_string file.file_id) ::
     ("peer_id", Sha1.direct_to_string !!client_uid) ::
     ("port", string_of_int !!client_port) ::
-    ("uploaded", Int64.to_string file.file_uploaded) ::
-    ("downloaded", Int64.to_string downloaded) ::
+    ("uploaded", Int64.to_string file.file_session_uploaded) ::
+    ("downloaded", Int64.to_string file.file_session_downloaded) ::
     ("left", Int64.to_string left) ::
     args
   in
-  
 
   let enabled_trackers =
     let enabled_trackers = List.filter (fun t -> tracker_is_enabled t) file.file_trackers in
@@ -137,18 +152,18 @@ let connect_trackers file event f =
     else begin
       (* if there is no tracker left, do something ? *)
       if !verbose_msg_servers then
-	lprintf_nl "No trackers left for %s, reenabling all of them..." (file_best_name (as_file file));
+        lprintf_nl "No trackers left for %s, reenabling all of them..." (file_best_name (as_file file));
       List.iter (fun t ->
-	match t.tracker_status with
-      (* only re-enable after normal error *)
+        match t.tracker_status with
+        (* only re-enable after normal error *)
         | Disabled _ -> t.tracker_status <- Enabled
-	| _ -> ()) file.file_trackers;
+        | _ -> ()) file.file_trackers;
       let enabled_trackers = List.filter (fun t -> tracker_is_enabled t) file.file_trackers in
       if enabled_trackers = [] && (file_state file) <> FilePaused then
-	begin
-	  file_pause (as_file file) (CommonUserDb.admin_user ());
-	  lprintf_file_nl (as_file file) "Paused %s, no usable trackers left" (file_best_name (as_file file))
-	end;
+      begin
+        file_pause (as_file file) (CommonUserDb.admin_user ());
+        lprintf_file_nl (as_file file) "Paused %s, no usable trackers left" (file_best_name (as_file file))
+      end;
       enabled_trackers
     end in
 
@@ -167,12 +182,12 @@ let connect_trackers file event f =
       then
         begin
           (* if we already tried to connect but failed, disable tracker, but allow re-enabling *)
-          if file.file_tracker_connected && t.tracker_last_clients_num = 0 &&
-            t.tracker_last_conn < 1 then begin
-              if !verbose_msg_servers then
-                lprintf_nl "Request error from tracker: disabling %s" t.tracker_url;
-	      t.tracker_status <- Disabled (intern "MLDonkey: Request error from tracker")
-            end
+          if file.file_tracker_connected && t.tracker_last_clients_num = 0 && t.tracker_last_conn < 1 then 
+          begin
+            if !verbose_msg_servers then
+              lprintf_nl "Request error from tracker: disabling %s" t.tracker_url;
+            t.tracker_status <- Disabled (intern "MLDonkey: Request error from tracker")
+          end
           (* Send request to tracker *)
           else begin
               let args = if String.length t.tracker_id > 0 then
@@ -182,7 +197,7 @@ let connect_trackers file event f =
                   ("key", t.tracker_key) :: args else args
               in
               if !verbose_msg_servers then
-                lprintf_nl "get_sources_from_tracker: tracker_connected:%s id:%s key:%s last_clients:%i last_conn-last_time:%i file: %s"
+                lprintf_nl "connect_trackers: tracker_connected:%s id:%s key:%s last_clients:%i last_conn-last_time:%i file: %s"
                   (string_of_bool file.file_tracker_connected)
                   t.tracker_id t.tracker_key t.tracker_last_clients_num
                   (t.tracker_last_conn - last_time()) file.file_name;
@@ -348,7 +363,7 @@ let disconnect_clients file =
 let download_finished file =
     if List.memq file !current_files then
       begin
-        connect_trackers file "completed" (fun _ _ -> ()); (*must be called before swarmer gets removed from file*)
+        connect_trackers file "completed" false (fun _ _ -> ()); (*must be called before swarmer gets removed from file*)
         (*CommonComplexOptions.file_completed*)
         file_completed (as_file file);
         (* Remove the swarmer for this file as it is not useful anymore... *)
@@ -405,25 +420,25 @@ let send_bitfield c =
       (
       match c.client_file.file_swarmer with
       | None ->
-	  (* This must be a seeded file... *)
-	  if !verbose_download then 
-	    lprintf_nl "Sending completed verified bitmap";
-	  let nchunks = Array.length c.client_file.file_chunks in
-	  let len = (nchunks+7)/8 in
-	  let s = String.make len '\000' in
-	  for i = 0 to nchunks - 1 do
-	    set_bit s i
-	  done;
-	  s
+          (* This must be a seeded file... *)
+          if !verbose_download then 
+            lprintf_nl "Sending completed verified bitmap";
+          let nchunks = Array.length c.client_file.file_chunks in
+          let len = (nchunks+7)/8 in
+          let s = String.make len '\000' in
+          for i = 0 to nchunks - 1 do
+            set_bit s i
+          done;
+          s
       | Some swarmer ->
-	  let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
-	  if !verbose_download then 
-	    lprintf_nl "Sending verified bitmap: [%s]" (VB.to_string bitmap);
-	  let len = (VB.length bitmap + 7)/8 in
-	  let s = String.make len '\000' in
-	  VB.iteri (fun i c ->
+          let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
+          if !verbose_download then 
+            lprintf_nl "Sending verified bitmap: [%s]" (VB.to_string bitmap);
+          let len = (VB.length bitmap + 7)/8 in
+          let s = String.make len '\000' in
+          VB.iteri (fun i c ->
             if c = VB.State_verified then set_bit s i) bitmap;
-	  s
+          s
     ))
 
 let counter = ref 0
@@ -673,7 +688,7 @@ and get_from_client sock (c: client) =
         match c.client_chunk with
         | None -> lprintf "none"
         | Some (chunk, blocks) -> List.iter (fun b -> 
-	    CommonSwarming.print_block b.up_block) blocks;
+            CommonSwarming.print_block b.up_block) blocks;
 
         lprint_newline ();
       
@@ -699,10 +714,10 @@ and get_from_client sock (c: client) =
               let chunk, blocks = CommonSwarming.find_blocks up in
               if !verbose_swarming then begin 
                 lprintf_n "Blocks Found: "; List.iter (fun b ->
-		  CommonSwarming.print_block b.up_block) blocks;
+                  CommonSwarming.print_block b.up_block) blocks;
                 lprint_newline ()
               end;
-	      c.client_chunk <- Some (chunk, blocks);
+              c.client_chunk <- Some (chunk, blocks);
 
              (*We put the found block in client_block to
                request range in this block. (Useful for
@@ -715,7 +730,7 @@ and get_from_client sock (c: client) =
 
               if !verbose_swarming then begin
                 lprintf_n "Current Blocks: "; List.iter (fun b ->
-		  CommonSwarming.print_block b.up_block) blocks;
+                  CommonSwarming.print_block b.up_block) blocks;
                 lprint_newline ()
               end;
 
@@ -723,7 +738,7 @@ and get_from_client sock (c: client) =
                 (*Given a block find a range inside*)
                 let (x,y,r) =
                   match c.client_range_waiting with
-		  | Some (x,y,r) ->
+                  | Some (x,y,r) ->
                         c.client_range_waiting <- None;
                         (x,y,r)
                   | None -> 
@@ -758,7 +773,7 @@ and get_from_client sock (c: client) =
                     lprintf_nl "Could not find range in current block";
 (*                  c.client_blocks <- List2.removeq b c.client_blocks; *)
 
-		  c.client_chunk <- None;
+                  c.client_chunk <- None;
 
                   iter ()
         in
@@ -860,6 +875,8 @@ and client_to_client c sock msg =
             count_download c (new_downloaded -- old_downloaded);
             (* use len here with max_dr quickfix *)
             Rate.update c.client_downloaded_rate ~amount:len;
+            (* count bytes downloaded from network for this file *)
+            file.file_session_downloaded <- file.file_session_downloaded ++ (Int64.of_int len);
             if !verbose_msg_clients then
               (match c.client_ranges_sent with
                   [] -> lprintf_file_nl (as_file file) "EMPTY Ranges !!!"
@@ -925,9 +942,9 @@ and client_to_client c sock msg =
                   if is_bit_set p i then begin
                     c.client_new_chunks <- i :: c.client_new_chunks;
                     match VB.get bitmap i with
-		    | VB.State_missing | VB.State_partial ->
+                    | VB.State_missing | VB.State_partial ->
                       c.client_interesting <- true
-		    | VB.State_complete | VB.State_verified -> ()
+                    | VB.State_complete | VB.State_verified -> ()
                   end 
                 done;
 
@@ -959,13 +976,13 @@ and client_to_client c sock msg =
               let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
               (* lprintf_nl "verified: %c;" (VB.state_to_char (VB.get bitmap n)); *)
               (* if the peer has a chunk we don't, tell him we're interested and update his bitmap *)
-	      match VB.get bitmap n with
-	      | VB.State_missing | VB.State_partial ->
+              match VB.get bitmap n with
+              | VB.State_missing | VB.State_partial ->
                   c.client_interesting <- true;
                   send_interested c;
                   c.client_new_chunks <- n :: c.client_new_chunks;
                   update_client_bitmap c;
-	      | VB.State_complete | VB.State_verified -> ()
+              | VB.State_complete | VB.State_verified -> ()
 
 (*        begin
           match c.client_bitmap, c.client_uploader with
@@ -1313,177 +1330,170 @@ let chk_keyval key n url name =
      0
    end
 
+(** Check that client is valid and record it *)
+let maybe_new_client file id ip port =
+  let cc = Geoip.get_country_code_option ip in
+  if id <> !!client_uid
+     && ip != Ip.null
+     && port <> 0
+     && (match !Ip.banned (ip, cc) with
+         | None -> true
+         | Some reason ->
+           if !verbose_connect then
+             lprintf_file_nl (as_file file) "%s:%d blocked: %s" (Ip.to_string ip) port reason;
+           false)
+  then
+    ignore (new_client file id (ip,port) cc);
+    if !verbose_sources > 1 then
+      lprintf_file_nl (as_file file) "Received %s:%d" (Ip.to_string ip) port;
+    ()
 
-(** In this function we initiate a connection to the file tracker
-  to get sources.
+let exn_catch f x = try `Ok (f x) with exn -> `Exn exn
+
+(** In this function we interact with the tracker
   @param file The file for which we want some sources
-  @param url Url of the tracker
-  If we have less than !!ask_tracker_threshold sources
-  and if we respect the file_tracker_interval then
-  we really ask sources to the tracker
+  @param need_sources whether we need any sources
 *)
-let get_sources_from_tracker file =
+let talk_to_tracker file need_sources =
+  (* This is the function which will be called by the http client for parsing the response *)
   let f t filename =
-    (*This is the function which will be called by the http client
-      for parsing the response
-     *)
-    let tracker_reply =
-      try
-        File.to_string filename
-      with e -> lprintf_file_nl (as_file file) "Empty reply from tracker"; ""
+    let tracker_failed reason =
+      (* On failure, disable the tracker and count attempts (@see is_tracker_enabled) *)
+      let num = match t.tracker_status with | Disabled_failure (i,_) -> i + 1 | _ -> 1 in
+      t.tracker_status <- Disabled_failure (num, intern reason);
+      lprintf_file_nl (as_file file) "Failure no. %d%s from Tracker %s for file: %s Reason: %s"
+        num
+        (if !!tracker_retries = 0 then "" else Printf.sprintf "/%d" !!tracker_retries)
+        t.tracker_url file.file_name (Charset.Locale.to_utf8 reason)
     in
-    let v =
-       match tracker_reply with
-       | "" ->
-        if !verbose_connect then
-          lprintf_file_nl (as_file file) "Empty reply from tracker";
-        Bencode.decode ""
-       | _ -> Bencode.decode tracker_reply
-    in
-    t.tracker_interval <- 600;
-    t.tracker_min_interval <- 600;
-    t.tracker_last_clients_num <- 0;
-    match v with
-      Dictionary list ->
+    match exn_catch File.to_string filename with
+    | `Exn _ | `Ok "" -> tracker_failed "empty reply"
+    | `Ok s ->
+    match exn_catch Bencode.decode s with
+    | `Exn exn -> tracker_failed (Printf.sprintf "wrong reply (%s)" (Printexc2.to_string exn))
+    | `Ok (Dictionary list) ->
+        t.tracker_interval <- 600;
+        t.tracker_min_interval <- 600;
+        if need_sources then t.tracker_last_clients_num <- 0;
+        let chk_keyval key n = chk_keyval key n t.tracker_url file.file_name in
+        if not (List.mem_assoc "failure reason" list) then
+        begin
+          begin match t.tracker_status with
+          | Disabled_failure (i, _) ->
+              lprintf_file_nl (as_file file) "Received good message from Tracker %s after %d bad attempts" 
+                t.tracker_url i
+          | _ -> () end;
+          (* Received good message from tracker after failures, re-enable tracker *)
+          t.tracker_status <- Enabled;
+        end;
         List.iter (fun (key,value) ->
-            (match (key, value) with
-            | String "failure reason", _ -> ()
-            | _ -> (match t.tracker_status with
-                   | Disabled_failure (i, _) ->
-                        lprintf_file_nl (as_file file) "Received good message from Tracker %s in file: %s after %d bad attempts"
-                          t.tracker_url file.file_name i
-                   | _ -> ());
-                   (* Received good message from tracker after failures, re-enable tracker *)
-                   t.tracker_status <- Enabled);
-
-            match (key, value) with
-            | String "failure reason", String failure ->
-                (* On failure, disable the tracker, count the attempts and forbid re-enabling *)
-		t.tracker_status <- (match t.tracker_status with
-                  | Disabled_failure (i,_) -> Disabled_failure (i + 1, intern failure)
-                  | _ -> Disabled_failure (1, intern failure));
-                lprintf_file_nl (as_file file) "Failure no. %d%s from Tracker %s in file: %s Reason: %s"
-                  (match t.tracker_status with | Disabled_failure (i,_) -> i | _ -> 1)
-                  (if !!tracker_retries = 0 then "" else Printf.sprintf "/%d" !!tracker_retries)
-                  t.tracker_url file.file_name (Charset.to_utf8 failure)
-            | String "warning message", String warning ->
-                lprintf_file_nl (as_file file) "Warning from Tracker %s in file: %s Reason: %s" t.tracker_url file.file_name warning
-            | String "interval", Int n ->
-                t.tracker_interval <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name;
+            match (key,value) with
+            | "failure reason", String failure -> tracker_failed failure
+            | "warning message", String warning ->
+                lprintf_file_nl (as_file file) "Warning from Tracker %s in file: %s Reason: %s" 
+                  t.tracker_url file.file_name warning
+            | "interval", Int n ->
+                t.tracker_interval <- chk_keyval key n;
                 (* in case we don't receive "min interval" *)
                 if t.tracker_min_interval > t.tracker_interval then
                   t.tracker_min_interval <- t.tracker_interval
-            | String "min interval", Int n ->
-                t.tracker_min_interval <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name;
+            | "min interval", Int n ->
+                t.tracker_min_interval <- chk_keyval key n;
                 (* make sure "min interval" is always < or equal to "interval" *)
                 if t.tracker_min_interval > t.tracker_interval then
                   t.tracker_min_interval <- t.tracker_interval
-            | String "downloaded", Int n ->
-                t.tracker_torrent_downloaded <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name
-            | String "complete", Int n
-            | String "done peers", Int n ->
-                t.tracker_torrent_complete <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name
-            | String "incomplete", Int n ->
-                t.tracker_torrent_incomplete <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name;
+            | "downloaded", Int n ->
+                t.tracker_torrent_downloaded <- chk_keyval key n
+            | "complete", Int n
+            | "done peers", Int n ->
+                t.tracker_torrent_complete <- chk_keyval key n
+            | "incomplete", Int n ->
+                t.tracker_torrent_incomplete <- chk_keyval key n;
                 (* if complete > 0 and we receive incomplete we probably won't receive num_peers so we simulate it below *)
                 if t.tracker_torrent_complete > 0 then
                   t.tracker_torrent_total_clients_count <- (t.tracker_torrent_complete + t.tracker_torrent_incomplete);
-            | String "num peers", Int n ->
-                t.tracker_torrent_total_clients_count <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name;
+            | "num peers", Int n ->
+                t.tracker_torrent_total_clients_count <- chk_keyval key n;
                 (* if complete > 0 and we receive num_peers we probably won't receive incomplete so we simulate it below *)
                 if t.tracker_torrent_complete > 0 then
                   t.tracker_torrent_incomplete <- (t.tracker_torrent_total_clients_count - t.tracker_torrent_complete);
-            | String "last", Int n ->
-                t.tracker_torrent_last_dl_req <- chk_keyval (Bencode.print key) n t.tracker_url file.file_name
-            | String "key", String n ->
+            | "last", Int n ->
+                t.tracker_torrent_last_dl_req <- chk_keyval key n
+            | "key", String n ->
                 t.tracker_key <- n;
                 if !verbose_msg_clients then
                   lprintf_file_nl (as_file file) "%s in file: %s has key: %s" t.tracker_url file.file_name n
-            | String "tracker id", String n ->
+            | "tracker id", String n ->
                 t.tracker_id <- n;
                 if !verbose_msg_clients then
                   lprintf_file_nl (as_file file) "%s in file: %s has tracker id %s" t.tracker_url file.file_name n
 
-            | String "peers", List list ->
+            | "peers", List list ->
+                if need_sources then
                 List.iter (fun v ->
                     match v with
-                      Dictionary list ->
-                        t.tracker_last_clients_num <- t.tracker_last_clients_num + 1;
-
+                    | Dictionary list ->
                         let peer_id = ref Sha1.null in
                         let peer_ip = ref Ip.null in
                         let port = ref 0 in
 
                         List.iter (fun v ->
                             match v with
-                              String "peer id", String id ->
+                              "peer id", String id ->
                                 peer_id := Sha1.direct_of_string id;
-                            | String "ip", String ip ->
+                            | "ip", String ip ->
                                 peer_ip := Ip.of_string ip
-                            | String "port", Int p ->
+                            | "port", Int p ->
                                 port := Int64.to_int p
                             | _ -> ()
                         ) list;
 
-                        (* Only record valid clients *)
-                        let cc = Geoip.get_country_code_option !peer_ip in
-                        if !peer_id != Sha1.null &&
-                          !peer_id <> !!client_uid &&
-                          !peer_ip != Ip.null &&
-                          !port <> 0 &&
-                          (match !Ip.banned (!peer_ip, cc) with
-                              None -> true
-                            | Some reason ->
-                                if !verbose_connect then
-                                  lprintf_file_nl (as_file file) "%s:%d blocked: %s"
-                                    (Ip.to_string !peer_ip) !port reason;
-                                false)
-                        then
-                          ignore (new_client file !peer_id (!peer_ip,!port) cc);
-                          if !verbose_sources > 1 then
-                            lprintf_file_nl (as_file file) "Received %s:%d"
-                              (Ip.to_string !peer_ip) !port;
-                          ()
+                        t.tracker_last_clients_num <- t.tracker_last_clients_num + 1;
+                        maybe_new_client file !peer_id !peer_ip !port
+
                     | _ -> assert false
                 ) list
-            | String "peers", String p ->
+            | "peers", String p ->
                 let rec iter_comp s pos l =
                   if pos < l then
                     let ip = Ip.of_ints (get_uint8 s pos,get_uint8 s (pos+1),
                         get_uint8 s (pos+2),get_uint8 s (pos+3))
                     and port = get_int16 s (pos+4)
                     in
-                    ignore (new_client file Sha1.null (ip,port) None);
                     t.tracker_last_clients_num <- t.tracker_last_clients_num + 1;
+                    maybe_new_client file Sha1.null ip port;
 
                     iter_comp s (pos+6) l
                 in
-                iter_comp p 0 (String.length p)
-            | String "private", Int n -> ()
+                if need_sources then 
+                  iter_comp p 0 (String.length p)
+            | "private", Int n -> ()
               (* TODO: if set to 1, disable peer exchange *)
 
-            | _ -> lprintf_file_nl (as_file file) "received unknown entry in answer from tracker: %s : %s" (Bencode.print key) (Bencode.print value)
+            | key, _ -> lprintf_file_nl (as_file file) "received unknown entry in answer from tracker: %s : %s" key (Bencode.print value)
         ) list;
        (*Now, that we have added new clients to a file, it's time
          to connect to them*)
         if !verbose_sources > 0 then
-          lprintf_file_nl (as_file file) "get_sources_from_tracker: got %i source(s) for file %s"
+          lprintf_file_nl (as_file file) "talk_to_tracker: got %i source(s) for file %s"
             t.tracker_last_clients_num file.file_name;
-        resume_clients file
+        if need_sources then resume_clients file
 
-    | _ -> assert false
+    | _ -> tracker_failed "wrong reply (value)" 
   in
   let event =
     if file.file_tracker_connected then ""
     else "started"
   in
-  connect_trackers file event f
+  connect_trackers file event need_sources f
 
 
 (** Check to see if file is finished, if not
   try to get sources for it
 *)
 let recover_files () =
+  if !verbose_share then
+    lprintf_nl "recover_files";
   List.iter (fun file ->
       match file.file_swarmer with
         None -> ()
@@ -1491,12 +1501,16 @@ let recover_files () =
           (try check_finished swarmer file with e -> ());
           match file_state file with
             FileDownloading ->
-              (try get_sources_from_tracker file with _ -> ())
+              if !verbose_share then
+                lprintf_file_nl (as_file file) "recover downloading";
+              (try talk_to_tracker file true with _ -> ())
           | FileShared ->
-              (try
-                  connect_trackers file "" (fun _ _ -> ()) with _ -> ())
+              if !verbose_share then
+                lprintf_file_nl (as_file file) "recover shared";
+              (try talk_to_tracker file false with _ -> ())
           | FilePaused -> () (*when we are paused we do nothing, not even logging this vvvv*)
-          | s -> lprintf_file_nl (as_file file) "Other state %s!!" (string_of_state s)
+          | FileQueued -> ()
+          | s -> if !verbose then lprintf_file_nl (as_file file) "recover: Other state %s!!" (string_of_state s)
       ) !current_files
 
 let upload_buffer = String.create 100000
@@ -1531,6 +1545,7 @@ let rec iter_upload sock c =
           Rate.update c.client_upload_rate  ~amount:len;
           Rate.update c.client_downloaded_rate;
           file.file_uploaded <- file.file_uploaded ++ (Int64.of_int len);
+          file.file_session_uploaded <- file.file_session_uploaded ++ (Int64.of_int len);
           let _ =
             (* update stats *)
             count_filerequest c;
@@ -1545,8 +1560,9 @@ let rec iter_upload sock c =
 (*          lprintf "sending piece\n"; *)
           send_client c (Piece (num, pos, upload_buffer, 0, len));
           iter_upload sock c
-  with e -> if !verbose then lprintf_nl
-		    "Exception %s in iter_upload" (Printexc2.to_string e)
+        with e -> 
+          if !verbose then 
+            lprintf_nl "Exception %s in iter_upload" (Printexc2.to_string e)
         end else
         begin
 (*          lprintf "client is waiting for another piece\n"; *)
@@ -1581,10 +1597,10 @@ let client_can_upload c allowed =
 let file_resume file =
   List.iter (fun t ->
     match t.tracker_status with
-      Enabled -> ()
-    | _ -> t.tracker_status <- Enabled
+    | Enabled | Disabled_mld _ -> ()
+    | Disabled_failure _ | Disabled _ -> t.tracker_status <- Enabled
   ) file.file_trackers;
-  (try get_sources_from_tracker file  with _ -> ())
+  (try talk_to_tracker file true with _ -> ())
 
 
 
@@ -1595,7 +1611,7 @@ let file_resume file =
 let file_stop file =
     if file.file_tracker_connected then
     begin
-      connect_trackers file "stopped" (fun _ _ ->
+      connect_trackers file "stopped" false (fun _ _ ->
           lprintf_file_nl (as_file file) "Tracker return: stopped %s" file.file_name;
           file.file_tracker_connected <- false)
     end
