@@ -194,7 +194,7 @@ let check_if_interesting file c =
       (match c.client_chunk with
       | None -> true
       | Some (chunk, blocks) ->
-	  List.for_all (fun b ->
+          List.for_all (fun b ->
             let chunk_num = CommonSwarming.block_chunk_num swarmer b.up_block in
             let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
             VB.get bitmap chunk_num <> VB.State_verified) blocks)
@@ -235,12 +235,12 @@ let can_handle_tracker t =
 
 let rec set_trackers file file_trackers =
   match file_trackers with
-    | [] -> ()
-    | url :: q ->
-	if not (List.exists (fun tracker -> 
-			       tracker.tracker_url = url
-			    ) file.file_trackers) then 
-	  let t = {
+  | [] -> ()
+  | url :: q ->
+        if not (List.exists (fun tracker -> 
+                               tracker.tracker_url = url
+                            ) file.file_trackers) then 
+          let t = {
             tracker_url = url;
             tracker_interval = 600;
             tracker_min_interval = 600;
@@ -253,12 +253,12 @@ let rec set_trackers file file_trackers =
             tracker_torrent_last_dl_req = 0;
             tracker_id = "";
             tracker_key = "";
-	    tracker_status = Enabled
+                  tracker_status = Enabled
           } in
-	  if not (can_handle_tracker t) then
-	    t.tracker_status <- Disabled_mld (intern "Tracker type not supported");
-	  file.file_trackers <-  t :: file.file_trackers;
-	set_trackers file q
+          if not (can_handle_tracker t) then
+            t.tracker_status <- Disabled_mld (intern "Tracker type not supported");
+          file.file_trackers <-  t :: file.file_trackers;
+          set_trackers file q
 
 let new_file file_id t torrent_diskname file_temp file_state user group =
   try
@@ -287,6 +287,8 @@ let new_file file_id t torrent_diskname file_temp file_state user group =
           file_torrent_diskname = torrent_diskname;
           file_completed_hook = (fun _ -> ());
           file_shared = None;
+          file_session_uploaded = Int64.zero;
+          file_session_downloaded = Int64.zero;
         } and file_impl =  {
           dummy_file_impl with
           impl_file_owner = user;
@@ -325,7 +327,7 @@ let new_file file_id t torrent_diskname file_temp file_state user group =
                           if not (Bitv.get bitmap num) then
                             send_client c (Have (Int64.of_int num));
                           check_if_interesting file c
-                    end       
+                    end
               ) file.file_clients
 
           );
@@ -410,9 +412,9 @@ let dot_string_of_string s =
   String.iter (fun s ->
     match s with
     | '0' .. '9' ->
-	if !found_non_int then Buffer.add_char buf '.';
-	found_non_int := false;
-	Buffer.add_char buf s
+        if !found_non_int then Buffer.add_char buf '.';
+        found_non_int := false;
+        Buffer.add_char buf s
     | _ -> found_non_int := true
   ) s;
   Buffer.contents buf
@@ -465,6 +467,7 @@ let decode_az_style s =
       | "ZT" -> Brand_ziptorrent
       | "bk" -> Brand_bitkitten
       | "MP" -> Brand_moopolice
+      | "UM" -> Brand_utorrent_mac
       | "UT" -> Brand_utorrent
       | "KT" -> Brand_ktorrent
       | "LP" -> Brand_lphant
@@ -482,9 +485,15 @@ let decode_az_style s =
       | "AX" -> Brand_bitpump
       | "DE" -> Brand_deluge
       | "TT" -> Brand_tuotu
+      | "SD" (* Thunder (aka XùnLéi) *)
       | "XL" -> Brand_xunlei
       | "FT" -> Brand_foxtorrent
       | "BF" -> Brand_bitflu
+      | "OS" -> Brand_oneswarm
+      | "LW" -> Brand_limewire
+      | "HL" -> Brand_halite
+      | "MR" -> Brand_miro
+      | "PD" -> Brand_pando
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
@@ -493,12 +502,15 @@ let decode_az_style s =
           (* 4.56 *)
           | Brand_bitpump
           | Brand_bitcomet -> (String.sub s 4 1) ^ "." ^ (String.sub s 5 2)
-          (* 3.45 *)
+          (* 3.4.5 *)
           | Brand_tuotu
+          | Brand_utorrent_mac
+          | Brand_oneswarm
           | Brand_utorrent -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "." ^ (String.sub s 5 1)
+          (* 3.45 *)
+          | Brand_transmission -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 2)
           (* 34.56 *)
-          | Brand_ctorrent
-          | Brand_transmission -> (strip_leading_zeroes (String.sub s 3 2)) ^ "." ^ (strip_leading_zeroes(String.sub s 5 2))
+          | Brand_ctorrent -> (strip_leading_zeroes (String.sub s 3 2)) ^ "." ^ (strip_leading_zeroes(String.sub s 5 2))
           (* 3.4.5->[R=RC.6|D=Dev|''] *)
           | Brand_ktorrent -> 
               let x = match s.[5] with 
@@ -594,6 +606,7 @@ let decode_simple_style s =
       (0,  "btpd",              Brand_btpd,            (dot_string(String.sub s 5 3)));
       (0,  "XBT",               Brand_xbt,             (dot_string(String.sub s 3 3)));
       (0,  "-FG",               Brand_flashget,        (dot_string(String.sub s 4 3)));
+      (0,  "-SP",               Brand_bitspirit,       (dot_string(String.sub s 3 3)));
     ]
   in
   let len = List.length !simple_list in
