@@ -61,26 +61,58 @@ let interpret_azureus_porttest s =
     let value = decode s in 
     match value with
     | Dictionary alist ->
-	(try
-	   match List.assoc (String "result") alist with
-	   | Int 1L -> "Port test OK!"
-	   | Int 0L ->
-	       (try
-		  match List.assoc (String "reason") alist with
-		  | String reason -> failure_message "%s" reason
-		  | _ -> raise Not_found
-		with Not_found ->
-		  failure_message "%s" "no reason given")
-	   | Int status ->
-	       failure_message "unknown status code (%Ld)" status
-	   | _ -> raise Not_found
-	 with Not_found ->
-	   failure_message "%s" "no status given")
+        (try
+           match List.assoc "result" alist with
+           | Int 1L -> "Port test OK!"
+           | Int 0L ->
+               (try
+                  match List.assoc "reason" alist with
+                  | String reason -> failure_message "%s" reason
+                  | _ -> raise Not_found
+                with Not_found ->
+                  failure_message "%s" "no reason given")
+           | Int status ->
+               failure_message "unknown status code (%Ld)" status
+           | _ -> raise Not_found
+         with Not_found ->
+           failure_message "%s" "no status given")
     | _ ->
-	failure_message "unexpected value type %s" (Bencode.print value)
+        failure_message "unexpected value type %s" (Bencode.print value)
   with _ ->
     failure_message "%s" "broken bencoded value"
 
+let interpret_utorrent_porttest s =
+  if String2.contains s "<div class=\"status-image\">OK!</div>" then
+    "Port test OK!"
+  else
+    "Port is not accessible"
+
+let perform_porttests tests =
+  match tests with
+  | [] -> porttest_result := PorttestResult (last_time(), "No tests available")
+  | _ ->
+  let module H = Http_client in
+  porttest_result := PorttestInProgress (last_time ());
+  let rec loop = function
+  | [] -> ()
+  | (url,interpret)::other ->
+    let r = {
+      H.basic_request with
+      H.req_url = Url.of_string url;
+      H.req_user_agent = get_user_agent ();
+      (* no sense in using proxy anyway *)
+(*       H.req_proxy = !CommonOptions.http_proxy; *)
+      H.req_max_total_time = 45.;
+    } in
+    H.wget_string r 
+      (fun s -> porttest_result := PorttestResult (last_time (), interpret s))
+      ~ferr:(fun code -> 
+        porttest_result := PorttestResult (last_time (), Printf.sprintf "Remote service error (%d)" code);
+        loop other)
+      (fun _ _ -> ())
+  in
+  loop tests
+
 let op_file_all_sources file =
   let list = ref [] in
   Hashtbl.iter (fun _ c ->
@@ -153,26 +185,30 @@ let op_file_commit file new_name =
       match file.file_shared with
       | None -> ()
       | Some old_impl ->
-	begin
-	  let impl = {
-	    impl_shared_update = 1;
-	    impl_shared_fullname = file_disk_name file;
-	    impl_shared_codedname = old_impl.impl_shared_codedname;
-	    impl_shared_size = file_size file;
-	    impl_shared_id = Md4.null;
-	    impl_shared_num = 0;
-	    impl_shared_uploaded = old_impl.impl_shared_uploaded;
-	    impl_shared_ops = shared_ops;
-	    impl_shared_val = file;
-	    impl_shared_requests = old_impl.impl_shared_requests;
-	    impl_shared_file = Some (as_file file);
-	    impl_shared_servers = [];
-	  } in
-	  file.file_shared <- Some impl;
+        begin
+          let impl = {
+            impl_shared_update = 1;
+            impl_shared_fullname = file_disk_name file;
+            impl_shared_codedname = old_impl.impl_shared_codedname;
+            impl_shared_size = file_size file;
+            impl_shared_id = Md4.null;
+            impl_shared_num = 0;
+            impl_shared_uploaded = old_impl.impl_shared_uploaded;
+            impl_shared_ops = shared_ops;
+            impl_shared_val = file;
+            impl_shared_requests = old_impl.impl_shared_requests;
+            impl_shared_file = Some (as_file file);
+            impl_shared_servers = [];
+          } in
+          file.file_shared <- Some impl;
           replace_shared old_impl impl;
-	end
+        end
     end 
 
+let auto_links =
+  let re = Str.regexp_case_fold "\\(https?://[a-zA-Z0-9_.!~*'();/?:@&=+$,%-]+\\)" in
+  fun s -> Str.global_replace re "\\<a href=\\\"\\1\\\"\\>\\1\\</a\\>" s
+
 let op_file_print file o =
 
   let buf = o.conn_buf in
@@ -182,6 +218,11 @@ let op_file_print file o =
     ("Filename", "sr br", "Filename");
     ("", "sr", file.file_name) ];
 
+  Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+  html_mods_td buf [
+    ("Torrent metadata hash", "sr", "Hash");
+    ("", "sr", Sha1.to_hexa file.file_id) ];
+
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Search for other possible Torrent Files", "sr br", "Torrent Srch");
@@ -224,14 +265,14 @@ let op_file_print file o =
     ("Comment", "sr br", "Comment");
     ("", "sr", match file.file_comment with
         "" -> "-"
-      | _ -> file.file_comment) ];
+      | s -> auto_links s) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Created by", "sr br", "Created by");
     ("", "sr", match file.file_created_by with
         "" -> "-"
-      | _ -> file.file_created_by) ];
+      | s -> auto_links s) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -243,7 +284,7 @@ let op_file_print file o =
     ("Modified by", "sr br", "Modified by");
     ("", "sr", match file.file_modified_by with
         "" -> "-"
-      | _ -> file.file_modified_by) ];
+      | s -> auto_links s) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -261,66 +302,75 @@ let op_file_print file o =
     match l with
       | [] -> ()
       | t :: q ->
-	  if not (tracker_is_enabled t) then print_first_tracker q
-	  else begin
-	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-	    html_mods_td buf [
-              ("Last Connect", "sr br", "Last Connect");
+          if not (tracker_is_enabled t) then print_first_tracker q
+          else begin
+            Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+            html_mods_td buf [
+              ("Last Tracker Announce", "sr br", "Last Announce");
               ("", "sr", string_of_date t.tracker_last_conn) ];
 
-	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-	    html_mods_td buf [
-              ("Connect Interval", "sr br", "Con Interval");
-              ("", "sr", Printf.sprintf "%d" t.tracker_interval) ];
+            if t.tracker_last_conn > 1 then
+            begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+              ("Next Tracker Announce (planned)", "sr br", "Next Announce");
+              ("", "sr", string_of_date (t.tracker_last_conn + t.tracker_interval)) ];
+            end;
 
-	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-	    html_mods_td buf [
-              ("Connect Min Interval", "sr br", "Con Min Interval");
-              ("", "sr", Printf.sprintf "%d" t.tracker_min_interval) ];
-	    (* show only interesting answers*)
-	    if t.tracker_torrent_downloaded > 0 then begin
+            Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+            html_mods_td buf [
+              ("Tracker Announce Interval", "sr br", "Announce Interval");
+              ("", "sr", Printf.sprintf "%d seconds" t.tracker_interval) ];
+
+            Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+            html_mods_td buf [
+              ("Minimum Tracker Announce Interval", "sr br", "Min Announce Interval");
+              ("", "sr", Printf.sprintf "%d seconds" t.tracker_min_interval) ];
+
+            (* show only interesting answers*)
+            if t.tracker_torrent_downloaded > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Downloaded", "sr br", "Downloaded");
-		("", "sr", Printf.sprintf "%d" t.tracker_torrent_downloaded) ]
+                ("Downloaded", "sr br", "Downloaded");
+                ("", "sr", Printf.sprintf "%d" t.tracker_torrent_downloaded) ]
             end;
-	    if t.tracker_torrent_complete > 0 then begin
+            if t.tracker_torrent_complete > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Complete (seeds)", "sr br", "Complete");
-		("", "sr", Printf.sprintf "%d" t.tracker_torrent_complete) ]
+                ("Complete (seeds)", "sr br", "Complete");
+                ("", "sr", Printf.sprintf "%d" t.tracker_torrent_complete) ]
             end;
-	    if t.tracker_torrent_incomplete > 0 then begin
+            if t.tracker_torrent_incomplete > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Incomplete (peers)", "sr br", "Incomplete");
-		("", "sr", Printf.sprintf "%d" t.tracker_torrent_incomplete) ]
+                ("Incomplete (peers)", "sr br", "Incomplete");
+                ("", "sr", Printf.sprintf "%d" t.tracker_torrent_incomplete) ]
             end;
-	    if t.tracker_torrent_total_clients_count > 0 then begin
+            if t.tracker_torrent_total_clients_count > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Total client count", "sr br", "All clients");
-		("", "sr", Printf.sprintf "%d" t.tracker_torrent_total_clients_count) ]
+                ("Total client count", "sr br", "All clients");
+                ("", "sr", Printf.sprintf "%d" t.tracker_torrent_total_clients_count) ]
             end;
-	    if t.tracker_torrent_last_dl_req > 0 then begin
+            if t.tracker_torrent_last_dl_req > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Latest torrent request", "sr br", "Latest request");
-		("", "sr", Printf.sprintf "%ds" t.tracker_torrent_last_dl_req) ]
+                ("Latest torrent request", "sr br", "Latest request");
+                ("", "sr", Printf.sprintf "%ds" t.tracker_torrent_last_dl_req) ]
             end;
-	    if String.length t.tracker_id > 0 then begin
+            if String.length t.tracker_id > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Tracker id", "sr br", "Tracker id");
-		("", "sr", t.tracker_id) ]
+                ("Tracker id", "sr br", "Tracker id");
+                ("", "sr", t.tracker_id) ]
             end;
-	    if String.length t.tracker_key > 0 then begin
+            if String.length t.tracker_key > 0 then begin
               Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
-		("Tracker key", "sr br", "Tracker key");
-		("", "sr", t.tracker_key) ]
-	    end 
-	  end in
+                ("Tracker key", "sr br", "Tracker key");
+                ("", "sr", t.tracker_key) ]
+            end 
+          end in
   print_first_tracker file.file_trackers;
 
   (* This is bad.  Magic info should be automatically filled in when 
@@ -366,7 +416,8 @@ let op_file_print file o =
     ];
     incr cntr;
   ) file.file_files
-  end else begin
+  end (* use_html_mods *)
+  else begin
 
   Printf.bprintf buf "Trackers:\n";
   List.iter (fun tracker ->
@@ -391,7 +442,7 @@ let op_file_print file o =
     incr cntr;
     let magic_string =
       match magic with 
-	None -> ""
+        None -> ""
       | Some m -> Printf.sprintf " / %s" m;
     in
     Printf.bprintf buf "File %d: %s (%Ld bytes)%s\n" !cntr filename size magic_string
@@ -408,9 +459,9 @@ let op_file_print_sources file o =
     else
       (* List *)
       List.iter (fun (t,c,d)  ->
-	  (* Title Class Value *)
-	  Printf.bprintf buf "%s "
-	  d;
+          (* Title Class Value *)
+          Printf.bprintf buf "%s "
+          d;
       ) l
   in
   let html_mods_table_header buf n c l =
@@ -418,13 +469,13 @@ let op_file_print_sources file o =
       html_mods_table_header buf n c l
     else
       if List.length l > 0 then begin
-	Printf.bprintf buf "\n";
-	List.iter (fun (w,x,y,z)  ->
-	  (* Sort Class Title Value *)
-	  Printf.bprintf buf "%s "
-	  z;
-	) l;
-	Printf.bprintf buf "\n"
+        Printf.bprintf buf "\n";
+        List.iter (fun (w,x,y,z)  ->
+          (* Sort Class Title Value *)
+          Printf.bprintf buf "%s "
+          z;
+        ) l;
+        Printf.bprintf buf "\n"
     end
   in
 
@@ -468,19 +519,19 @@ let op_file_print_sources file o =
 *)
         ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
           (match file.file_swarmer with
-	  | None -> 0
-	  | Some swarmer ->
-	      let bitmap = 
-		CommonSwarming.chunks_verified_bitmap swarmer in
-	      VB.fold_lefti (fun acc _ s ->
-		if s = VB.State_verified then acc + 1 else acc)	0 bitmap)))
+          | None -> 0
+          | Some swarmer ->
+              let bitmap = 
+                CommonSwarming.chunks_verified_bitmap swarmer in
+              VB.fold_lefti (fun acc _ s ->
+                if s = VB.State_verified then acc + 1 else acc)        0 bitmap)))
       ] in
 
       html_mods_table_header buf "sourcesTable" "sources al" header_list;
 
       Hashtbl.iter (fun _ c ->
           let cinfo = client_info (as_client c) in
-	  if use_html_mods o then
+          if use_html_mods o then
           Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
 
           let btos b = if b then "T" else "F" in
@@ -533,7 +584,7 @@ let op_file_print_sources file o =
 
           html_mods_td buf td_list;
           if use_html_mods o then Printf.bprintf buf "\\</tr\\>"
-	  else Printf.bprintf buf "\n";
+          else Printf.bprintf buf "\n";
 
       ) file.file_clients;
 
@@ -673,7 +724,7 @@ let load_torrent_string s user group =
     lprintf_nl "Starting torrent download with diskname: %s"
         torrent_diskname;
   let file = new_download file_id torrent torrent_diskname user group in
-  BTClients.get_sources_from_tracker file;
+  BTClients.talk_to_tracker file true;
   CommonInteractive.start_download (file_find (file_num file));
   file
 
@@ -688,6 +739,7 @@ let load_torrent_file filename user group =
     Sys.remove filename;
   ignore (load_torrent_string s user group)
 
+(*
 let parse_tracker_reply file t filename =
 (*This is the function which will be called by the http client
 for parsing the response*)
@@ -723,6 +775,7 @@ for parsing the response*)
           | _ -> ()
       ) list;
   | _ -> assert false
+*)
 
 let try_share_file torrent_diskname =
   if !verbose_share then lprintf_nl "try_share_file: %s" torrent_diskname;
@@ -754,9 +807,11 @@ let try_share_file torrent_diskname =
     let user = CommonUserDb.admin_user () in
     let file = new_file file_id torrent torrent_diskname
         filename FileShared user user.user_default_group in
-    if !verbose_share then lprintf_file_nl (as_file file) "Sharing file %s" filename;
-    BTClients.connect_trackers file "started"
-      (parse_tracker_reply file)
+
+    if !verbose_share then 
+      lprintf_file_nl (as_file file) "Sharing file %s" filename;
+    BTClients.talk_to_tracker file false;
+    `Ok torrent_diskname
   with
   | Not_found ->
       (* if the torrent is still there while the file is gone, remove the torrent *)
@@ -765,21 +820,25 @@ let try_share_file torrent_diskname =
         Filename.concat old_directory
           (Filename.basename torrent_diskname)
       in
-      (try
+      begin try
           Unix2.rename torrent_diskname new_torrent_diskname;
-        with _ ->
-          (lprintf_nl "Failed to rename %s to %s"
-              torrent_diskname new_torrent_diskname));
+          `Ok new_torrent_diskname
+      with _ ->
+          let msg = Printf.sprintf "Failed to rename %S to %S" torrent_diskname new_torrent_diskname in
+          lprintf_nl "%s" msg;
+          `Err msg
+      end
   | e ->
-      lprintf_nl "Cannot share torrent %s for %s"
-        torrent_diskname (Printexc2.to_string e)
+      let msg = Printf.sprintf "Cannot share %S - exn %s" torrent_diskname (Printexc2.to_string e) in
+      lprintf_nl "%s" msg;
+      `Err msg
 
 (* Call one minute after start, and then every 20 minutes. Should
   automatically contact the tracker. *)
 let share_files _ =
   if !verbose_share then lprintf_nl "share_files";
   List.iter (fun file ->
-    try_share_file (Filename.concat seeded_directory file)
+    ignore (try_share_file (Filename.concat seeded_directory file))
   ) (Unix2.list_directory seeded_directory);
   let shared_files_copy = !current_files in
  (* if the torrent is gone while the file is still shared, remove the share *)
@@ -796,6 +855,10 @@ let share_files _ =
         end
   ) shared_files_copy
 
+(** talk_to_tracker maintains timers and will connect to trackers only when allowed by rules *)
+let announce_shared_files () =
+  List.iter (fun file -> if file_state file = FileShared then BTClients.talk_to_tracker file false) !current_files
+
 let scan_new_torrents_directory () =
   let filenames = Unix2.list_directory new_torrents_directory in
   List.iter (fun file ->
@@ -813,12 +876,12 @@ let scan_new_torrents_directory () =
       (try Sys.remove file with _ -> ())
     with 
       Torrent_can_not_be_used _ ->
-	Unix2.rename file (Filename.concat old_directory file_basename);
-	lprintf_nl "Torrent %s does not have valid tracker URLs, moved to torrents/old ..." file_basename
+        Unix2.rename file (Filename.concat old_directory file_basename);
+        lprintf_nl "Torrent %s does not have valid tracker URLs, moved to torrents/old ..." file_basename
     | e ->
-	Unix2.rename file (Filename.concat old_directory file_basename);
-	lprintf_nl "Error %s in scan_new_torrents_directory for %s, moved to torrents/old ..."
-	  (Printexc2.to_string e) file_basename
+        Unix2.rename file (Filename.concat old_directory file_basename);
+        lprintf_nl "Error %s in scan_new_torrents_directory for %s, moved to torrents/old ..."
+          (Printexc2.to_string e) file_basename
   ) filenames
 
 let retry_all_ft () =
@@ -876,7 +939,7 @@ let op_network_parse_url url user group =
                   ) "" cookies
                 ) ]
             with Not_found -> []);
-	  H.req_max_retry = 10;
+          H.req_max_retry = 10;
           } in
 
         let file_diskname = Filename.basename u.Url.short_file in
@@ -896,12 +959,12 @@ let op_network_parse_url url user group =
             load_torrent_file url user group;
             "", true
           with
-	    Torrent_already_exists _ -> "A torrent with this name is already in the download queue", false
-	  | Torrent_can_not_be_used _ -> "This torrent does not have valid tracker URLs", false
+            Torrent_already_exists _ -> "A torrent with this name is already in the download queue", false
+          | Torrent_can_not_be_used _ -> "This torrent does not have valid tracker URLs", false
         with e ->
           lprintf_nl "Exception %s while 2nd loading" (Printexc2.to_string e);
-	  let s = Printf.sprintf "Can not load load torrent file: %s"
-	    (Printexc2.to_string e) in
+          let s = Printf.sprintf "Can not load load torrent file: %s"
+            (Printexc2.to_string e) in
           s, false
       else
         begin
@@ -911,7 +974,7 @@ let op_network_parse_url url user group =
     | e ->
        lprintf_nl "Exception %s while loading" (Printexc2.to_string e);
        let s = Printf.sprintf "Can not load load torrent file: %s"
-	 (Printexc2.to_string e) in
+         (Printexc2.to_string e) in
        s, false
 
 let op_client_info c =
@@ -1007,32 +1070,69 @@ let op_client_dprint_html c o file str =
 
 let op_network_connected _ = true
 
-
-let get_default_tracker () = 
-  if !!BTTracker.default_tracker = "" then
-     Printf.sprintf "http://%s:%d/announce"
-      (Ip.to_string (CommonOptions.client_ip None))
-      !!BTTracker.tracker_port 
-   else
-     !!BTTracker.default_tracker
-
 let compute_torrent filename announce comment = 
-  let announce = if announce = "" then get_default_tracker () else announce in
+  let announce = if announce = "" then BTTracker.get_default_tracker () else announce in
   if !verbose then lprintf_nl "compute_torrent: [%s] [%s] [%s]"
    filename announce comment;
-  let basename = Filename.basename filename in
-  let torrent = Filename.concat seeded_directory
-    (Printf.sprintf "%s.torrent" basename) in
+  let basename = Printf.sprintf "%s.torrent" (Filename.basename filename) in
+  let torrent = Filename.concat seeded_directory basename in
   let is_private = 0 in
-  BTTorrent.generate_torrent announce torrent comment (Int64.of_int is_private) filename;
-  try_share_file torrent
+  let file_id = BTTorrent.generate_torrent announce torrent comment (Int64.of_int is_private) filename in
+  match try_share_file torrent with 
+  | `Err msg -> failwith msg
+  | `Ok torrent_path ->
+    Filename.concat (Sys.getcwd ()) torrent_path,
+    try `Ok (BTTracker.track_torrent basename file_id) with exn -> `Err (Printexc2.to_string exn)
+
+(* let text fmt = Printf.ksprintf (fun s -> `Text s) fmt *)
+(* 
+  OCaml 3.08.3 compatibility (ksprintf not available)
+  http://mldonkey.sourceforge.net/phpBB2/viewtopic.php?p=30453 
+*)
+let text s = `Text s
+let link name url = `Link (name,url)
+
+let output buf typ elements =
+  let f = match typ with
+  | HTML | XHTML | XML ->
+    begin function 
+    | `Text s -> Xml.buffer_escape buf s
+    | `Link (name,url) -> 
+        Printf.bprintf buf "<a href=\"%s\">%s</a>" 
+          (Xml.escape url) (Xml.escape (match name with "" -> url | s -> s))
+    | `Break -> Buffer.add_string buf "<br/>"
+    end
+  | TEXT | ANSI ->
+    begin function
+    | `Text s -> Buffer.add_string buf s
+    | `Link ("",url) -> Printf.bprintf buf "%s" url
+    | `Link (name,url) -> Printf.bprintf buf "%s <%s>" name url
+    | `Break -> Buffer.add_string buf "\n"
+    end
+  in
+  List.iter f elements
+
+(* dirty hack *)
+let output o l =
+  match o.conn_output with
+  | ANSI | TEXT -> output o.conn_buf o.conn_output l
+  | HTML | XHTML | XML ->
+    let buf = Buffer.create 1024 in
+    output buf o.conn_output l;
+    let s = Buffer.contents buf in
+    for i = 0 to String.length s - 1 do
+      begin match s.[i] with
+      | '<' | '>' | '\\' | '"' | '&' -> Buffer.add_char o.conn_buf '\\'
+      | _ -> () end;
+      Buffer.add_char o.conn_buf s.[i]
+    done
 
 let commands =
 
     [
     "compute_torrent", "Network/Bittorrent", Arg_multiple (fun args o ->
-      let buf = o.conn_buf in
-      try
+      output o
+      begin try
         let filename = ref "" in
         let comment = ref "" in
         (match args with
@@ -1040,54 +1140,39 @@ let commands =
         | [fname] -> filename := fname
         | _ -> raise Not_found);
 
-        compute_torrent !filename "" !comment;
-
-        if o.conn_output = HTML then
-          (* TODO: really htmlize it *)
-          Printf.bprintf buf ".torrent file generated"
-        else
-          Printf.bprintf buf ".torrent file generated\n";
+        let (path,url) = compute_torrent !filename "" !comment in
+        [
+          text (Printf.sprintf "Torrent file generated : %s" path);
+          `Break;
+          (match url with
+          | `Ok url -> link "Download" url
+          | `Err s -> text (Printf.sprintf "Not tracked : %s" s));
+          `Break
+        ]
+      with 
+      | Not_found -> [text "Not enough parameters"; `Break]
+      | exn -> [text (Printf.sprintf "Error: %s" (Printexc2.to_string exn)); `Break]
+      end;
       ""
-      with Not_found ->
-          if o.conn_output = HTML then
-            (* TODO: really htmlize it *)
-            Printf.bprintf buf "Not enough parameters"
-          else
-            Printf.bprintf buf "Not enough parameters\n";
-      ""
-    ), _s "<filename> <comment> :\tgenerate the corresponding <filename> .torrent file with <comment> in torrents/tracked/.\n\t\t\t\t\tThe file is automatically tracked, and seeded if in incoming/";
+    ), _s "<filename> [<comment>] :\tgenerate the corresponding <filename> .torrent file with <comment>.\n\t\t\t\t\t\tThe file is automatically tracked if tracker is enabled and seeded if located in incoming/";
 
     "torrents", "Network/Bittorrent", Arg_none (fun o ->
-      let buf = o.conn_buf in
-      if !!BTTracker.tracker_port <> 0 then begin
-          Printf.bprintf o.conn_buf (_b ".torrent files available:\n");
+      output o 
+        begin try
+          BTTracker.check_tracker ();
           let files_tracked = Unix2.list_directory tracked_directory in
           let files_downloading = Unix2.list_directory downloads_directory in
           let files_seeded = Unix2.list_directory seeded_directory in
-          let all_torrents_files = files_tracked @ files_downloading @ files_seeded in
+          let files_old = Unix2.list_directory old_directory in
+          let all_torrents_files = files_tracked @ files_downloading @ files_seeded @ files_old in
 
-          if o.conn_output = HTML then
-            (* TODO: really htmlize it *)
-            List.iter (fun file ->
-                Printf.bprintf buf "http://%s:%d/%s "
-                  (Ip.to_string (CommonOptions.client_ip None))
-                !!BTTracker.tracker_port
-                  file
-            ) all_torrents_files
-          else
-            List.iter (fun file ->
-                Printf.bprintf buf "http://%s:%d/%s\n"
-                  (Ip.to_string (CommonOptions.client_ip None))
-                !!BTTracker.tracker_port
-                  file
-            ) all_torrents_files;
-        end
-      else
-          if o.conn_output = HTML then
-            (* TODO: really htmlize it *)
-            Printf.bprintf buf "Tracker not activated (tracker_port = 0)"
-          else
-            Printf.bprintf buf "Tracker not activated (tracker_port = 0)\n";
+          let l = List.map (fun file -> [link file (BTTracker.tracker_url file); `Break]) all_torrents_files in
+
+          (`Text (_s ".torrent files available:")) :: `Break :: List.flatten l
+        with
+          exn ->
+          [`Text (Printexc2.to_string exn); `Break]
+        end;
         _s ""
     ), _s ":\t\t\t\tprint all .torrent files on this server";
 
@@ -1124,13 +1209,14 @@ let commands =
       if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
       List.iter (fun file ->
           if file_state file = FileShared then
-              Printf.bprintf o.conn_buf "%s [%s]\n" file.file_name (Int64.to_string file.file_uploaded)
+              Printf.bprintf o.conn_buf "%s [U %Ld u/d %Ld/%Ld]\n" 
+                file.file_name file.file_uploaded file.file_session_uploaded file.file_session_downloaded
       ) !current_files;
       _s "done"
       end else
       begin print_command_result o "You are not allowed to use seeded_torrents";
       "" end
-    ), _s ":\t\t\tprint all seeded .torrent files on this server";
+    ), _s ":\t\t\tprint all seeded .torrent files on this server (output: name, total upload, session upload, session download)";
 
     "reshare_torrents", "Network/Bittorrent", Arg_none (fun o ->
       share_files ();
@@ -1165,15 +1251,15 @@ let commands =
       let buf = o.conn_buf in
       if Sys.file_exists url then
         begin
-	  load_torrent_file url o.conn_user.ui_user o.conn_user.ui_user.user_default_group;
+          load_torrent_file url o.conn_user.ui_user o.conn_user.ui_user.user_default_group;
           Printf.bprintf buf "loaded file %s\n" url
-	end
+        end
       else
         begin
           let url = "Location: " ^ url ^ "\nContent-Type: application/x-bittorrent" in
-	  let result = fst (op_network_parse_url url o.conn_user.ui_user o.conn_user.ui_user.user_default_group) in
+          let result = fst (op_network_parse_url url o.conn_user.ui_user o.conn_user.ui_user.user_default_group) in
           Printf.bprintf buf "%s\n" result
-	end;
+        end;
       _s ""
      ),  "<url|file> :\t\t\tstart BT download";
 
@@ -1255,17 +1341,17 @@ let op_gui_message s user =
         let file = load_torrent_string text user user.user_default_group in
         raise (Torrent_started file.file_name)
       with e -> (match e with
-	| Torrent_can_not_be_used s -> lprintf_nl "Loading torrent from GUI: torrent %s can not be used" s
-	| Torrent_already_exists s -> lprintf_nl "Loading torrent from GUI: torrent %s is already in download queue" s
-	| _ -> ());
-	raise e)
+        | Torrent_can_not_be_used s -> lprintf_nl "Loading torrent from GUI: torrent %s can not be used" s
+        | Torrent_already_exists s -> lprintf_nl "Loading torrent from GUI: torrent %s is already in download queue" s
+        | _ -> ());
+        raise e)
   | 1 -> (* 34+ *)
       let n = get_int s 2 in
       let a, pos = get_string s 6 in
       let c, pos = get_string s pos in
       let sf = CommonShared.shared_find n in
       let f = shared_fullname sf in
-      compute_torrent f a c;
+      ignore (compute_torrent f a c)
   | opcode -> failwith (Printf.sprintf "[BT] Unknown message opcode %d" opcode)
 
 let _ =
@@ -1331,22 +1417,13 @@ let _ =
     ]);
   network.op_network_porttest_result <- (fun _ -> !porttest_result);
   network.op_network_porttest_start <- (fun _ -> 
-      let module H = Http_client in
       azureus_porttest_random := (Random.int 100000);
-      porttest_result := PorttestInProgress (last_time ());
-      let r = {
-          H.basic_request with
-          H.req_url =
-	    Url.of_string (Printf.sprintf
-	      "http://azureus.aelitis.com/natcheck.php?port=%d&check=azureus_rand_%d"
-		!!client_port !azureus_porttest_random);
-          H.req_proxy = !CommonOptions.http_proxy;
-          H.req_user_agent = get_user_agent ();
-        } in
-      H.wget r (fun file ->
-	let result = interpret_azureus_porttest (File.to_string file) in
-	porttest_result := PorttestResult (last_time (), result)
-      )
+      let tests = [
+        Printf.sprintf "http://www.utorrent.com/testport?port=%d" !!client_port, interpret_utorrent_porttest;
+        Printf.sprintf "http://azureus.aelitis.com/natcheck.php?port=%d&check=azureus_rand_%d"
+          !!client_port !azureus_porttest_random, interpret_azureus_porttest;
+      ] in
+      perform_porttests tests
   );
   network.op_network_check_upload_slots <- (fun _ -> check_bt_uploaders ());
   client_ops.op_client_info <- op_client_info;
