@@ -69,10 +69,10 @@ let enable () =
         lprint_newline();
         lprintf_nl "BT-client_port and BT-tracker_port can not be the same.";
         lprintf_nl "Change one of the settings and restart MLDonkey, exiting...\n";
-	Pervasives.exit 69
+        Pervasives.exit 69
       end;
     if !!BTTracker.tracker_port > 0 then (
-        try BTTracker.start_tracker ()
+        try BTTracker.start_tracker !!tracked_files_list
         with e ->
             lprintf "Exception in BTTracker.start_tracker: %s\n"
               (Printexc2.to_string e));
@@ -84,6 +84,7 @@ let enable () =
     add_session_timer enabler !!import_new_torrents_interval (fun _ ->
         BTInteractive.scan_new_torrents_directory ();
     );
+    add_session_timer enabler 60. (fun _ -> BTInteractive.announce_shared_files (); );
     add_timer 5. (fun _ -> BTInteractive.scan_new_torrents_directory ());
     add_timer 10. BTInteractive.share_files;
     add_session_timer enabler 600. BTInteractive.retry_all_ft;
@@ -105,8 +106,8 @@ let enable () =
 
   CommonGlobals.do_at_exit ( fun _ ->
     List.iter (fun file ->
-		 BTClients.file_stop file
-	      ) !current_files;
+                 BTClients.file_stop file
+              ) !current_files;
   );
 
   BTClients.listen ();
