@@ -84,14 +84,14 @@ let send_key = define_option bittorrent_section ["send_key"]
   "Send client key to trackers"
     bool_option true
 
-let max_uploaders_per_torrent = define_option bittorrent_section ["max_uploaders_per_torrent"]
-  "Maximum number of uploaders for one torrent, can not be higher than max_bt_uploaders"
-    int_option 3
-
 let max_bt_uploaders = define_option bittorrent_section ["max_bt_uploaders"]
   "Maximum number of uploaders for bittorrent, can not be higher than max_upload_slots"
     int_option 3
 
+let max_uploaders_per_torrent = define_option bittorrent_section ["max_uploaders_per_torrent"]
+  "Maximum number of uploaders for one torrent, can not be higher than max_bt_uploaders"
+    int_option 3
+
 (* numwant: Optional. Number of peers that the client would like to receive from the tracker.
 This value is permitted to be zero. If omitted, typically defaults to 50 peers.   *)
 
@@ -120,13 +120,13 @@ let _ =
       (fun _ ->
         let v = int_of_string (strings_of_option max_uploaders_per_torrent).option_default in
         if !!max_uploaders_per_torrent < 1 then max_uploaders_per_torrent =:= v;
-	check_bt_uploaders ()
+        check_bt_uploaders ()
         );
     option_hook max_bt_uploaders
       (fun _ ->
         if !!max_bt_uploaders < 1 then
             max_bt_uploaders =:= int_of_string (strings_of_option max_bt_uploaders).option_default;
-	check_bt_uploaders ()
+        check_bt_uploaders ()
         );
     option_hook max_tracker_redirect   (** #4541 [egs] **)
       (fun _ ->
@@ -155,7 +155,12 @@ let old_files = define_option bittorrent_section ["old_files"]
   ""
     (list_option (tuple2_option (string_option, int64_option))) []
 
+let tracked_files_list = define_option bittorrent_section ["tracked_files"]
+  ""
+    (list_option Sha1.option) []
+
 let get_user_agent () = 
   if !!user_agent = "default" then
     CommonOptions.get_user_agent ()
   else !!user_agent
+
