@@ -283,13 +283,18 @@ let find_sockets_client sock =
   | Found_client c -> Some c
   | _ -> None )
 
-(* set our nick for hubs from .ini or global *)              
+(* set our nick for hubs from .ini or global *)
 let local_login () =
   if !!login = "" then !!CommonOptions.global_login else !!login
-    
+
 (* Shorten string to some maximum length *)
 let shorten_string s length =
-  if (String.length s > length) then String.sub s 0 (length-1)
+  if length < String.length s then
+    try
+      let n = Charset.utf8_nth s length in
+      String.sub s 0 n
+    with
+      _ -> s (* relies on bounds checking! FIXME? *)
   else s 
 
 (* Replace one string to another string from string *)
@@ -497,6 +502,17 @@ let new_upfile dcsh fd =
   } in
   file
 
+(* FIXME review *)
+let safe_filename s =
+  let s = String.copy s in
+  for i = 0 to String.length s - 1 do
+    match s.[i] with
+    | c when Char.code c < 32 -> s.[i] <- '_'
+    | '.' | '/' | '\\' | ':' -> s.[i] <- '_'
+    | _ -> ()
+  done;
+  s
+
 (* Return existing file or create new one *)     
 let new_file tiger_root (directory:string) (filename:string) (file_size:int64) =
   (try
@@ -511,13 +527,10 @@ let new_file tiger_root (directory:string) (filename:string) (file_size:int64) =
       if !verbose_download then lprintf_nl "File exists: (%s) (%s)" f.file_directory f.file_name;
       f 
   with _ ->
-      let temp_filename =
+      let temp_filename = safe_filename
         (match tiger_root with
-        | "" ->
-          let dname = ref (String.copy directory) in
-          String2.replace_char !dname '/' '_';  
-          Printf.sprintf "DC_%s_%s" !dname filename  
-        | _ -> Printf.sprintf "DC_%s" tiger_root )
+        | "" -> Printf.sprintf "DC_%s_%s" directory filename
+        | _ -> Printf.sprintf "DC_%s" tiger_root)
       in 
       let fullname = Filename.concat !!temp_directory temp_filename in
       let temp_file = Unix32.create_rw fullname in
@@ -588,7 +601,7 @@ let new_client () =
           dummy_client_impl with
           impl_client_val = c;
           impl_client_ops = client_ops;
-	  impl_client_upload = None;
+          impl_client_upload = None;
         } in
   (*lprintf_nl "New client";     *)
   CommonClient.new_client impl; 
@@ -955,7 +968,7 @@ let dc_with_connected_servers f =
 (* Return hub state text *)
 let dc_hubstate_to_text s =
   (match s.server_hub_state with
-  | Waiting -> "Waiting"
+  | Waiting -> "Not connected"
   | User -> "User"
   | Vipped -> "Vip"
   | Opped -> "Op" )
@@ -1022,38 +1035,38 @@ let new_result user tiger_root (directory:string) (filename:string) (filesize:in
   
 (*
 /** We don't keep leaves for blocks smaller than this... */
-	static const int64_t MIN_BLOCK_SIZE = 64*1024;
+        static const int64_t MIN_BLOCK_SIZE = 64*1024;
 *)
 (*CommonHasher.compute_tiger :
   string -> int64 -> int64 -> (Md4.TigerTree.t job -> unit) -> unit 
 *)
   
 (* DC++ blocksize
-		int64_t bl = 1024;
-		while(bl * (int64_t)d->getTigerTree().getLeaves().size() < d->getTigerTree().getFileSize())
-			bl *= 2;
-		d->getTigerTree().setBlockSize(bl);
-		d->getTigerTree().calcRoot();
+                int64_t bl = 1024;
+                while(bl * (int64_t)d->getTigerTree().getLeaves().size() < d->getTigerTree().getFileSize())
+                        bl *= 2;
+                d->getTigerTree().setBlockSize(bl);
+                d->getTigerTree().calcRoot();
 *)
 (*
 AdcCommand Download::getCommand(bool zlib, bool tthf) {
-	AdcCommand cmd(AdcCommand::CMD_GET);
-	if(isSet(FLAG_TREE_DOWNLOAD)) {
-		cmd.addParam("tthl");
-	} else if(isSet(FLAG_PARTIAL_LIST)) {
-		cmd.addParam("list");
-	} else {
-		cmd.addParam("file");
-	}
-	if(tthf && getTTH() != NULL) {
-		cmd.addParam("TTH/" + getTTH()->toBase32());
-	} else {
-		cmd.addParam(Util::toAdcFile(getSource()));
-	}
-	cmd.addParam(Util::toString(getPos()));
-	cmd.addParam(Util::toString(getSize() - getPos()));
+        AdcCommand cmd(AdcCommand::CMD_GET);
+        if(isSet(FLAG_TREE_DOWNLOAD)) {
+                cmd.addParam("tthl");
+        } else if(isSet(FLAG_PARTIAL_LIST)) {
+                cmd.addParam("list");
+        } else {
+                cmd.addParam("file");
+        }
+        if(tthf && getTTH() != NULL) {
+                cmd.addParam("TTH/" + getTTH()->toBase32());
+        } else {
+                cmd.addParam(Util::toAdcFile(getSource()));
+        }
+        cmd.addParam(Util::toString(getPos()));
+        cmd.addParam(Util::toString(getSize() - getPos()));
   
-	if(zlib && getSize() != -1 && BOOLSETTING(COMPRESS_TRANSFERS)) {
-		cmd.addParam("ZL1");
-	}
+        if(zlib && getSize() != -1 && BOOLSETTING(COMPRESS_TRANSFERS)) {
+                cmd.addParam("ZL1");
+        }
 *)
