@@ -171,7 +171,8 @@ let client_to_server s m sock =
   (match m with
   
   | BadPassReq ->
-      if !verbose_msg_servers then lprintf_nl "Bad password for server: %s" (Ip.string_of_addr s.server_addr);
+      if !verbose_msg_servers then 
+        lprintf_nl "Bad password for %S on %s" s.server_last_nick (Ip.string_of_addr s.server_addr);
       s.server_hub_state <- User
 
   | ConnectToMeReq t ->  (* client is unknown at this moment until $MyNick is received *)
@@ -189,7 +190,8 @@ let client_to_server s m sock =
           if !verbose_unexpected_messages then lprintf_nl "%s in ConnectToMe sending" (Printexc2.to_string e) )
   
   | ForceMoveReq t ->
-       disconnect_server s (Closed_for_error "Forcemove command received");
+      lprintf_nl "Received ForceMove(%S) from %s" t (Ip.string_of_addr s.server_addr);
+      disconnect_server s (Closed_for_error "Forcemove command received")
       
   | GetPassReq -> (* After password request from hub ... *)
       let addr = Ip.string_of_addr s.server_addr in
@@ -389,7 +391,7 @@ let client_to_server s m sock =
                 if !verbose_unknown_messages then lprintf_nl "Codedname was wrong in Search receiving";
                 raise Not_found )
             in
-            let message =                 (* message structure for both active and passive messages *)        
+            let message =                 (* message structure for both active and passive messages *)
               let module S = SR in {
                 S.owner = s.server_last_nick;
                 S.directory = directory;
@@ -411,11 +413,11 @@ let client_to_server s m sock =
         end;
             if t.Search.passive then 
               dc_send_msg sock (SRReq ( message ))
-            else             
+            else
               DcClients.udp_send (Ip.of_string t.Search.ip) (int_of_string t.Search.port) (SRReq ( message ))
           ) files
         in
-        
+
         if t.Search.passive then begin                          (* if passive search received *)
           if not !!firewalled then begin                        (* and we are in active mode  *)
             if (t.Search.nick <> s.server_last_nick) then begin (* if search is not from ourself ... *)
@@ -446,7 +448,7 @@ let client_to_server s m sock =
       let u = new_user (Some s) t.To.from in
       u.user_messages <- u.user_messages @ [
         (int_of_float (current_time ()), t.To.from, PrivateMessage (0, t.To.message))];
-      
+
   | UnknownReq m -> 
       if m <> "" then
         if !verbose_unexpected_messages || !verbose_msg_servers then
@@ -456,34 +458,38 @@ let client_to_server s m sock =
           else lprintf_nl "%s (%s)" txt m
 
   | UserCommandReq -> () (* Not supported atm *)
-  
+
   | UserIPReq st -> (* CHECK *)
       if !verbose_msg_servers then lprintf_nl "Received $UserIP";
-      List.iter ( fun nameip ->
-        lprintf_nl "UserIPReq: nameip=%s" nameip; 
-        match String2.split nameip ' ' with
-        | name :: ip :: [] -> 
-            (try
-              let u = search_user_by_name name in 
-              ( try u.user_ip <- Ip.addr_of_string ip with _ -> () );
-              lprintf_nl "Added ip %s to user %s" (Ip.string_of_addr u.user_ip) u.user_nick
-            with _ ->
-                if !verbose_unexpected_messages then lprintf_nl "No user by name %s" name )
-        | _ -> ()
-      ) st;
-                    
-  | ValidateDenideReq n ->  
+      let st = UserIP.parse_nameip st in
+      List.iter begin fun (name,addr) ->
+        lprintf_nl "UserIP: %s %s" name (Ip.string_of_addr addr); 
+        try
+          if name = s.server_last_nick then
+          begin
+            match addr with 
+            | Ip.AddrIp ip -> lprintf_nl "Received self IP: %s" (Ip.to_string ip); last_high_id := ip
+            | Ip.AddrName _ -> ()
+          end;
+          let u = search_user_by_name name in
+          u.user_ip <- addr;
+          lprintf_nl "Added ip %s to user %s" (Ip.string_of_addr u.user_ip) u.user_nick
+        with _ ->
+          if !verbose_unexpected_messages then lprintf_nl "No user by name %s" name
+      end st
+
+  | ValidateDenideReq n ->
       let errortxt = Printf.sprintf "Nick %s is already in use" n in
       if !verbose_unexpected_messages || !verbose_msg_servers then
         lprintf_nl "%s" errortxt;
       disconnect_server s (Closed_for_error errortxt )
-                    
+
   | VersionReq v -> ()
-                    
+
   | _ -> 
     lprintf_nl "--> Unhandled server message. Implement ?:";
     DcProtocol.dc_print m )
-                    
+
 (* connect to DC server *)
 let connect_server s =
   if can_open_connection connection_manager then
@@ -545,10 +551,10 @@ let make_hublist_from_file f =
             end;
             incr counter;
             let r = {
-              dc_name = Charset.to_utf8 server_name;  
+              dc_name = Charset.Locale.to_utf8 server_name;
               dc_ip = Ip.addr_of_string !addr;
               dc_port = !port;
-              dc_info = Charset.to_utf8 server_info;
+              dc_info = Charset.Locale.to_utf8 server_info;
               dc_nusers = !nusers;
             } in
             hublist := r :: !hublist
@@ -559,6 +565,56 @@ let make_hublist_from_file f =
   if !verbose_msg_servers then lprintf_nl "Found %d valid servers from hublist" !counter;
   !hublist
 
+let xml_tag name = let name = String.lowercase name in fun x -> String.lowercase (Xml.tag x) = name
+
+let rec xml_select names xs =
+  match names with
+  | [] -> xs
+  | [name] -> List.filter (xml_tag name) xs
+  | name::t ->
+    let l = List.filter (xml_tag name) xs in
+    xml_select t (List.concat (List.map Xml.children l))
+
+let ssplit s sub =
+  try
+    let n = String2.search_from s 0 sub in
+    Some (String2.before s n, String2.after s (n + String.length sub))
+  with
+    _ -> None
+
+let parse_address s =
+  let s = match ssplit (String.lowercase s) "://" with
+  | Some ("dchub",s) -> s
+  | None -> s
+  | Some (("adc"|"adcs"),_) -> failwith "ADC not supported"
+  | Some (proto,_) -> failwith (Printf.sprintf "Unsupported protocol %S" proto)
+  in
+  try Scanf.sscanf s "%s@:%u" (fun s n -> s,n) with _ -> s,411
+
+let make_hublist_from_xml x =
+  let make_hub x =
+    let name = Charset.Locale.to_utf8 (Xml.attrib x "Name") in
+    let (address,port) = parse_address (Xml.attrib x "Address") in
+    let info = Charset.Locale.to_utf8 (Xml.attrib x "Description") in
+    let nusers = int_of_string (Xml.attrib x "Users") in
+    {
+      dc_name = name;
+      dc_ip = Ip.addr_of_string address;
+      dc_port = port;
+      dc_info = info;
+      dc_nusers = nusers;
+    }
+  in
+  let l = xml_select ["hublist";"hubs";"hub"] [x] in
+  let add_hub acc x = 
+    try make_hub x :: acc
+    with exn -> lprintf_nl "Skipping hublist entry : %s" (Printexc2.to_string exn); acc
+  in
+  let l' = List.fold_left add_hub [] l in
+  if !verbose_msg_servers then
+    lprintf_nl "Found %u valid (of %u total) servers in hublist" (List.length l') (List.length l);
+  l'
+
 (* Connect to all autoconnect servers once *)  
 let autoconnect_to_servers () =
   Hashtbl.iter (fun _ s ->
