@@ -936,16 +936,16 @@ value ml_has_pthread(value unit)
 #define NTFS_SUPER_MAGIC      0x5346544E
 
 struct statfs {
-   long    f_type;     /* type of filesystem (see below) */
-   long    f_bsize;    /* optimal transfer block size */
-   long    f_blocks;   /* total data blocks in file system */
-   long    f_bfree;    /* free blocks in fs */
-   long    f_bavail;   /* free blocks avail to non-superuser */
-   long    f_files;    /* total file nodes in file system */
-   long    f_ffree;    /* free file nodes in fs */
-   long    f_fsid;     /* file system id */
-   long    f_namelen;  /* maximum length of filenames */
-   long    f_spare[6]; /* spare for later */
+   unsigned __int64    f_type;     /* type of filesystem (see below) */
+   unsigned __int64    f_bsize;    /* optimal transfer block size */
+   unsigned __int64    f_blocks;   /* total data blocks in file system */
+   unsigned __int64    f_bfree;    /* free blocks in fs */
+   unsigned __int64    f_bavail;   /* free blocks avail to non-superuser */
+   unsigned __int64    f_files;    /* total file nodes in file system */
+   unsigned __int64    f_ffree;    /* free file nodes in fs */
+   unsigned __int64    f_fsid;     /* file system id */
+   unsigned __int64    f_namelen;  /* maximum length of filenames */
+   unsigned __int64    f_spare[6]; /* spare for later */
 };
 
 static int statfs (const unsigned char *path, struct statfs *buf)
@@ -1483,3 +1483,25 @@ value ml_os_supported(value unit)
 	  return Val_false;
 }
 
+/*
+ * fsync
+ */
+CAMLprim value ml_fsync(value v_fd)
+{
+    CAMLparam1(v_fd);
+    int r = 0;
+    OS_FD fd;
+
+    if (OS_IS_FD(v_fd)) 
+      fd = Fd_val(v_fd); 
+    else 
+      caml_invalid_argument("fsync");
+
+    caml_enter_blocking_section();
+    r = os_fsync(fd);
+    caml_leave_blocking_section();
+    if (0 != r)
+      uerror("fsync",Nothing);
+    CAMLreturn(Val_unit);
+}
+
