@@ -21,6 +21,8 @@ type t
 val of_inet_addr : Unix.inet_addr -> t
 val of_string : string -> t
 val of_ints : int * int * int * int -> t
+val get_lo16: t -> int
+val get_hi16: t -> int
 
 val to_inet_addr : t -> Unix.inet_addr
 val to_string : t -> string
