@@ -59,6 +59,14 @@ type event =
 | READ_DONE of int
 | BASIC_EVENT of BasicSocket.event
 
+let string_of_event = function
+| CONNECTED -> "CONNECTED"
+| WRITE_DONE -> "WRITE_DONE"
+| CAN_REFILL -> "CAN_REFILL"
+| BUFFER_OVERFLOW -> "BUFFER_OVERFLOW"
+| READ_DONE n -> Printf.sprintf "READ_DONE %d" n
+| BASIC_EVENT e -> string_of_basic_event e
+
 type token = {
     mutable token_used : bool;
     connection_manager : connection_manager;
@@ -908,7 +916,16 @@ let bandwidth_controler t sock =
   (match t.read_control with
       None -> ()
     | Some bc ->
-        must_read sock (bc.total_bytes = 0 || bc.remaining_bytes > 0));
+(*         must_read sock (bc.total_bytes = 0 || bc.remaining_bytes > 0)); *)
+        (* bandwidth_controler is called before socket is engaged into data transfer.
+           This can be either upload or download connection, but when download speed
+           is capped and fully saturated the above condition will be false,
+           consequently the socket will never get [want_read] property and
+           will never get considered by the event loop, until finally being 
+           closed on timeout. This bug manifests itself with no _new_ BT upload clients
+           or inability to connect to servers (DC) when download is running.
+           This is a temporary fix, bandwidth limiting logic needs some global refactoring *)
+        must_read sock true);
   (match t.write_control with
       None -> ()
     | Some bc ->
